Forex.Cache.DETS (Forex v1.1.2)

Copy Markdown View Source

Implementation of the Forex.Cache behaviour using DETS.

Summary

Functions

Returns true if all given keys have non-expired entries in the cache for the given ttl (in milliseconds).

Functions

warm?(keys, ttl)

@spec warm?(keys :: [any()], ttl :: non_neg_integer()) :: boolean()

Returns true if all given keys have non-expired entries in the cache for the given ttl (in milliseconds).

This can be used to determine whether a startup fetch should be skipped because the persisted cache is still fresh.