This module defines the behaviour of the parser that will be used to parse the XML response from the European Central Bank (ECB).
Summary
Callbacks
Parse the XML response body from the European Central Bank (ECB).
This module defines the behaviour of the parser that will be used to parse the XML response from the European Central Bank (ECB).
Parse the XML response body from the European Central Bank (ECB).