Supervisor for the Forex.Fetcher module that fetches exchange rates.
The Supervisor provides functionality to start, stop and restarting the Forex.Fetcher.
The Forex.Fetcher.Supervisor accepts the following options:
auto_start- A boolean value that determines if the fetcher process should be started automatically when the supervisor is started. The default value istrue.use_cache- A boolean value that determines if the cache should be used. The default value istrue.
Summary
Functions
Returns a specification to start this module under a supervisor.
Delete the Forex exchange rate fetcher process from the supervisor.
Check if the Forex exchange rate fetcher process has been initiated.
Check if the Forex exchange rate fetcher process is running.
Get the status of the Forex exchange rate fetcher process.
If the process is running, it returns :running,
if it has been initiated but not running, it returns :stopped,
otherwise, it returns :not_started.
Restart the stoped Forex exchange rate fetcher process.
Start the Forex exchange rate fetcher process.
Starts the Forex.Fetcher.Supervisor process.
Stop the Forex exchange rate fetcher process.
Functions
Returns a specification to start this module under a supervisor.
See Supervisor.
Delete the Forex exchange rate fetcher process from the supervisor.
Check if the Forex exchange rate fetcher process has been initiated.
Check if the Forex exchange rate fetcher process is running.
Get the status of the Forex exchange rate fetcher process.
If the process is running, it returns :running,
if it has been initiated but not running, it returns :stopped,
otherwise, it returns :not_started.
Restart the stoped Forex exchange rate fetcher process.
Start the Forex exchange rate fetcher process.
Starts the Forex.Fetcher.Supervisor process.
This function should be called in your application supervision tree to start the Forex exchange rate fetcher.
Options
auto_start- A boolean value that determines if the fetcher process should be started automatically when the supervisor is started. The default value istrue.use_cache- A boolean value that determines if the cache should be used. The default value istrue.name- The name to register the supervisor process under. Defaults toForex.Fetcher.Supervisor.
Stop the Forex exchange rate fetcher process.