Options/configuration module for the Forex.Fetcher.Supervisor and the rates functions
(latest_rates/1, last_ninety_days_rates/1, etc.) in the Forex module.
Summary
Functions
Validate and return the options for the Forex.Currency module functions.
Validate and return the options for the Forex.Fetcher module functions. Supported options
Validate and return the options for the Forex.Fetcher.Supervisor module functions. Supported options
Validate and return the options for the Forex module functions.
Types
Functions
@spec currency_options(opts :: keyword()) :: validated_options :: [currency_option()] | map()
Validate and return the options for the Forex.Currency module functions.
Supported options:
:format- Format of rate values The default value is:decimal.:round- Decimal places for rounding The default value is5.
@spec fetcher_options(opts :: keyword()) :: validated_options :: [fetcher_option()] | map()
Validate and return the options for the Forex.Fetcher module functions. Supported options:
@spec fetcher_supervisor_options(opts :: keyword()) :: validated_options :: [fetcher_supervisor_option()] | map()
Validate and return the options for the Forex.Fetcher.Supervisor module functions. Supported options:
:auto_start(boolean/0) - Whether to start the fetcher automatically The default value istrue.
@spec rates_options(opts :: keyword()) :: validated_options :: [rates_option()] | map()
Validate and return the options for the Forex module functions.
Supported options:
:base- The base currency to convert rates to The default value is:eur.:format- Format of rate values The default value is:decimal.:round- Decimal places for rounding The default value is5.:keys- Map key format The default value is:atoms.:symbols- Currency codes to include:use_cache(boolean/0) - Whether to use cached rates The default value istrue.:feed_fn- Optional custom feed function as{module, function, args}